# Lupus Decoupled Form Support

## Overview

The module makes it easy to support Drupal forms in a decoupled frontend. Together with the Nuxtjs Custom Elements
Renderer module, it provides a ready-to-use solution for supporting Drupal form in a progressive way. The form HTML
is rendered by Drupal and wrapped in a <drupal-form> custom element, such that the frontend can easily add some
styling or custom JS-enhancements.

Atm, only progressive form submissions are supported, which work without JavaScript. For optional JavaScript enhanced
submissions there is some WIP code at https://www.drupal.org/project/lupus_decoupled/issues/3471135

## How does it work?

Generally, Drupal renders the form as usual. The resulting render array and thus HTML is wrapped into
a `<drupal-form>` component, which simply shows the server-rendered HTML of the form. When the form is submitted,
the frontend takes care of forwarding the POST request to the backend, where it's processed as usual. After form
processing the request is rendered in a custom element response again: Either it's a page response with a
reloaded form or a redirect. Any messages for form validation fails or success are handled as usual via the Drupal
messages system and sent as part of the next page response.
Thus, the frontend needs to take care of rendering the response appropriately in a new page or redirect, just as it
does for any other server-render page.

### Frontend support

* For the Nuxt frontend, The Nuxtjs Drupal-CE Connector module ships with a suiting component:
  https://github.com/drunomics/nuxtjs-drupal-ce/blob/2.x/playground/components/global/DrupalForm.vue
  Moreover, there is server middleware which takes care of handling the POST request:
  https://github.com/drunomics/nuxtjs-drupal-ce/blob/4d8c9e43d8a3ed1f1f6480425b25016b82e57579/src/runtime/server/middleware/drupalFormHandler.ts


### Theming forms

For having a clean, semantic output of forms, the lupus-stark theme has been created - which
adds suiting Drupal twig files to do away unnecessary Drupal-isms. That way decent markup is
generated and necessary style adaptions are made possible by targeting elements using the
reasonable default classes.
The module adds support for a special '_theme' key in route definitions, what allows switching
themes to the given theme-name specifically for the form routes, if preferred.

## Adding support for a form

For adding a new form a custom-elements enabled form processing route must be added. This is
done easily by:

1. Ensuring there is a custom-elements enabled route, e.g. by cloning the original one and
   adding `custom_elements` as `_format`.
2. Pointing the custom-elements enabled route to a form controller that renders the results as
   custom elements. For standard form or entity_form controllers the following controllers may be specified:
     * For general forms: `lupus_decoupled_form.controller.form:getContentResult`
     * For entity forms:  `lupus_decoupled_form.controller.entity_form:getContentResult`
   Else, a custom controller can be easily be provided with the help of the `CustomElementsFormControllerTrait`,
   see the `lupus_decoupled_contact ` for an example.

That's it!

Generally, https://git.drupalcode.org/project/lupus_decoupled/-/tree/1.x/modules/ serves as
a simple example that demonstrates how support for contact forms is enabled.
